'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _elements = require('./elements');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * getDecorator
 * @param decoratorName
 * @returns {object|null}
 */
var getDecorator = function getDecorator(decoratorName) {

    var decorators = this.decorators || [];

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(decorators), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var decorator = _step.value;
            var name = decorator.expression.callee.name;

            if (decoratorName === name) {
                return decorator;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return null;
};

/**
 * getElementClassByName
 * @param name {string}
 * @param elements {object}
 * @returns {*|string}
 */
var getElementClassByName = function getElementClassByName(name, elements) {

    var element = elements[name] || 'HTMLElement';
    return element;
};

/**
 * getArguments
 * @param t {object}
 * @returns {{extends: string}}
 */
var getArguments = function getArguments(t) {

    // set default
    var args = {
        extends: 'div'
    };

    var _expression$arguments = (0, _slicedToArray3.default)(this.expression.arguments, 1),
        firstArgument = _expression$arguments[0];

    if (!t.isObjectExpression(firstArgument)) {
        return args;
    }

    var properties = firstArgument.properties;
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = (0, _getIterator3.default)(properties), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var property = _step2.value;

            args[property.key.name] = property.value.value;
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return args;
};

/**
 * addSuperClass
 * @param element {string}
 * @param t {object}
 */
var addSuperClass = function addSuperClass(element, t) {

    var identifier = t.identifier(element);
    this.superClass = identifier;
};

/**
 * add static string property
 * @param prop {string}
 * @param value {string}
 * @param t {object}
 */
var addStaticStringProperty = function addStaticStringProperty(prop, value, t) {

    var a = t.classProperty(t.identifier(prop), t.stringLiteral(value));
    a.static = true;
    this.body.body.push(a);
};

/**
 * addStaticGetterProperty
 * @param type {string}
 * @param superClass {string}
 * @param t {object}
 */
var addStaticGetterProperty = function addStaticGetterProperty(type, superClass, t) {

    var classMethodNode = t.classMethod('get', t.identifier(type), [], t.blockStatement([t.returnStatement(t.stringLiteral(superClass))]), false, true);

    this.body.body.push(classMethodNode);
};

/**
 * getProgram
 * @param node
 * @returns {*}
 */
var getProgram = function getProgram(node) {
    while (node.parent) {
        node = node.findParent(function (node) {
            return node;
        });
        if (node.key === "program") {
            return node;
        }
    }
};

/**
 * plugin
 * @astexplorer:
 *  - https://astexplorer.net/#/gist/844e84f3b94e1b4b3a4e1e30b9abc8d2/934cc82a41d787ba43ec9413a570be7639b305b4
 * @param t {types}
 * @returns {{visitor: {ClassDeclaration: (function(*=))}}}
 */
var plugin = function plugin(_ref) {
    var t = _ref.types;


    return {
        pre: function pre() {
            this.cache = new _map2.default();
        },
        post: function post() {
            this.cache.clear();
        },

        visitor: {
            Program: function Program(path) {
                var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : state,
                    opts = _ref2.opts;

                // shim elementToFunction
                if (!opts.elementToFunc) {
                    return;
                }

                var programBody = path.node.body;
                var importSrcPath = "app-decorators/src/libs/element-to-function";

                // return when import for elementToFunc already exists
                var _iteratorNormalCompletion3 = true;
                var _didIteratorError3 = false;
                var _iteratorError3 = undefined;

                try {
                    for (var _iterator3 = (0, _getIterator3.default)(programBody), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                        var node = _step3.value;

                        if (node.type !== "ImportDeclaration") continue;
                        if (node.source.value === importSrcPath) return;
                    }
                } catch (err) {
                    _didIteratorError3 = true;
                    _iteratorError3 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion3 && _iterator3.return) {
                            _iterator3.return();
                        }
                    } finally {
                        if (_didIteratorError3) {
                            throw _iteratorError3;
                        }
                    }
                }

                var name = this.cache.get("elementToFunc");
                if (!name) {
                    name = path.scope.generateUidIdentifier("elementToFunc").name;
                    this.cache.set("elementToFunc", name);
                }

                // create import statement for elementToFunc
                var importDeclaration = t.importDeclaration([t.importDefaultSpecifier(t.identifier(name))], t.stringLiteral(importSrcPath));

                // add import statement for elementToFunc
                programBody.unshift(importDeclaration);
            },

            /**
             * ClassDeclaration
             * @param node {object}
             * @param opts {object}
             */
            ClassDeclaration: function ClassDeclaration(path) {
                var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : state,
                    opts = _ref3.opts;

                var options = opts || {};
                var node = path.node;


                var component = getDecorator.call(node, 'component');
                if (!component) {

                    /**
                     * If no @component but HTML... as superClass
                     * and no extends() then add addStaticGetterProperty
                     */
                    return;
                }

                var className = node.id.name;
                addStaticStringProperty.call(node, '$$componentName', className, t);

                var superClass = getArguments.call(component, t).extends;
                var element = getElementClassByName(superClass, _elements.elements);
                addSuperClass.call(node, element, t);

                if (!/div/.test(superClass)) {
                    addStaticGetterProperty.call(node, 'extends', superClass, t);
                }

                // shim elementToFunction
                if (!options.elementToFunc) {
                    return;
                }

                /**
                 * Begin for elementToFunc shim
                 */

                // replace extends identifier with "elementToFunc"
                var name = this.cache.get("elementToFunc");

                var callExpressionNode = t.callExpression(t.identifier(name), [t.identifier(element)]);
                path.node.superClass = callExpressionNode;
            }
        }
    };
};

exports.default = plugin;